<!DOCTYPE html>
<html>
<head>
<title>Error</title>
<style>
    body {
        width: 35em;
        margin: 0 auto;
        font-family: Tahoma, Verdana, Arial, sans-serif;
    }
</style>
</head>
<body>
<h1>An error occurred in {{NAME}}.</h1>
<p>This error was logged and will be followed by by the {{NAME}} maintainers.<br/>
Please try again later.</p>
</body>
</html>
